@@ -11,7 +11,7 @@ def add_lengths(g, x, y):
     for i, e in enumerate(g.es()):
         lon1, lat1 = g.vs[e.source][x], g.vs[e.source][y]
         lon2, lat2 = g.vs[e.target][x], g.vs[e.target][y]
-        g.es[i]['length'] = geo.haversine(lon1, lat1, lon2, lat2)
+        g.es[i]['length'] = geo.haversine([lon1, lat1], [lon2, lat2])
     return g
 
 ##########################################################
